/*
 * Author: Jake Goodwin
 * Date: 2023
 * Description: A USART library for the atmega series of chips
 * Filename: avr_usart.h
 */
//#define __AVR_ATmega328P__
#include <inttypes.h>
#include <stdint.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

//#############################
//PIN DEFINES AND CONSTS
//#############################

/*
 * AVR D0 --> AT-09 TX
 * AVR D1 --> AT-09 RX
 * AVR D2 --> AT-09 State
 */


//F_CPU and BAUD should be defined with -D when compiling the code.
#ifndef F_CPU
    #define F_CPU   14745600UL
#endif

#define F_UBRR     ((F_CPU)/(BAUD*16UL)-1)

#define TX_PIN      PIND1
#define TX_DDR      DDD1

#define RX_PIN      PIND0
#define RX_DDR      DDD0

#define AT_PORT     PORTD

//LED PIN for indicator and debug.
#define LED_PIN     PINC0
#define LED_DDR     DDC0


//#############################
//MACROS
//#############################

#define USE_U2X 0
//#define CARRIAGE_RETURN 1
//#define LINE_FEED 1

#define DEBUG  1

#if DEBUG == 1
#define debug(data, data_len) serial0_write(data, data_len)
#else
#define debug(data, data_len)
#endif

//#############################
//FUNCTION PROTOTYPES
//#############################
void init_usart0(void);
void tx_usart0( char data);
char rx_usart0(void);

void serial0_write( char* buffer, uint8_t write_length);
void serial0_read( char* buffer, uint8_t buf_length);
void serial0_set_baud(uint8_t baud);
void seiral0_enable_timeouts(uint8_t ms);
void serial0_flush_rxbuf(void);
void serial0_enable_parity_bit(uint8_t pos);
void serial0_set_stop_bits(uint8_t number);





